@@ -1,18 +1,18 @@
 /* Set file access and modification times.
 
-   Copyright (C) 2003-2024 Free Software Foundation, Inc.
+   Copyright (C) 2003-2019 Free Software Foundation, Inc.
 
-   This file is free software: you can redistribute it and/or modify
-   it under the terms of the GNU Lesser General Public License as
-   published by the Free Software Foundation, either version 3 of the
-   License, or (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by the
+   Free Software Foundation; either version 3 of the License, or any
+   later version.
 
-   This file is distributed in the hope that it will be useful,
+   This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU Lesser General Public License for more details.
+   GNU General Public License for more details.
 
-   You should have received a copy of the GNU Lesser General Public License
+   You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /* Written by Paul Eggert.  */
@@ -26,7 +26,7 @@
 
 #include <errno.h>
 #include <fcntl.h>
-#include <string.h>
+#include <stdbool.h>
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <unistd.h>
@@ -52,9 +52,7 @@
 
 /* Avoid recursion with rpl_futimens or rpl_utimensat.  */
 #undef futimens
-#if !HAVE_NEARLY_WORKING_UTIMENSAT
-# undef utimensat
-#endif
+#undef utimensat
 
 /* Solaris 9 mistakenly succeeds when given a non-directory with a
    trailing slash.  Force the use of rpl_stat for a fix.  */
@@ -78,21 +76,6 @@ static int utimensat_works_really;
 static int lutimensat_works_really;
 #endif /* HAVE_UTIMENSAT || HAVE_FUTIMENS */
 
-static bool
-is_valid_timespec (struct timespec const *timespec)
-{
-  return (timespec->tv_nsec == UTIME_NOW
-          || timespec->tv_nsec == UTIME_OMIT
-          || (0 <= timespec->tv_nsec && timespec->tv_nsec < TIMESPEC_HZ));
-}
-
-static bool
-is_valid_timespecs (struct timespec const timespec[2])
-{
-  return (is_valid_timespec (&timespec[0])
-          && is_valid_timespec (&timespec[1]));
-}
-
 /* Validate the requested timestamps.  Return 0 if the resulting
    timespec can be used for utimensat (after possibly modifying it to
    work around bugs in utimensat).  Return a positive value if the
@@ -105,7 +88,14 @@ validate_timespec (struct timespec timespec[2])
 {
   int result = 0;
   int utime_omit_count = 0;
-  if (!is_valid_timespecs (timespec))
+  if ((timespec[0].tv_nsec != UTIME_NOW
+       && timespec[0].tv_nsec != UTIME_OMIT
+       && ! (0 <= timespec[0].tv_nsec
+             && timespec[0].tv_nsec < TIMESPEC_HZ))
+      || (timespec[1].tv_nsec != UTIME_NOW
+          && timespec[1].tv_nsec != UTIME_OMIT
+          && ! (0 <= timespec[1].tv_nsec
+                && timespec[1].tv_nsec < TIMESPEC_HZ)))
     {
       errno = EINVAL;
       return -1;
@@ -133,14 +123,14 @@ validate_timespec (struct timespec timespec[2])
   return result + (utime_omit_count == 1);
 }
 
-/* Normalize any UTIME_NOW or UTIME_OMIT values in (*TS)[0] and (*TS)[1],
-   using STATBUF to obtain the current timestamps of the file.  If
+/* Normalize any UTIME_NOW or UTIME_OMIT values in *TS, using stat
+   buffer STATBUF to obtain the current timestamps of the file.  If
    both times are UTIME_NOW, set *TS to NULL (as this can avoid some
    permissions issues).  If both times are UTIME_OMIT, return true
    (nothing further beyond the prior collection of STATBUF is
    necessary); otherwise return false.  */
 static bool
-update_timespec (struct stat const *statbuf, struct timespec **ts)
+update_timespec (struct stat const *statbuf, struct timespec *ts[2])
 {
   struct timespec *timespec = *ts;
   if (timespec[0].tv_nsec == UTIME_OMIT
@@ -228,7 +218,7 @@ fdutimens (int fd, char const *file, struct timespec const timespec[2])
   if (0 <= utimensat_works_really)
     {
       int result;
-# if defined __linux__ || defined __sun || defined __NetBSD__
+# if __linux__ || __sun
       /* As recently as Linux kernel 2.6.32 (Dec 2009), several file
          systems (xfs, ntfs-3g) have bugs with a single UTIME_OMIT,
          but work if both times are either explicitly specified or
@@ -238,10 +228,9 @@ fdutimens (int fd, char const *file, struct timespec const timespec[2])
          where UTIME_OMIT would have worked.
 
          The same bug occurs in Solaris 11.1 (Apr 2013).
-         The same bug occurs in NetBSD 10.0 (May 2024).
 
-         FIXME: Simplify this in 2024, when these file system bugs are
-         no longer common on Gnulib target platforms.  */
+         FIXME: Simplify this for Linux in 2016 and for Solaris in
+         2024, when file system bugs are no longer common.  */
       if (adjustment_needed == 2)
         {
           if (fd < 0 ? stat (file, &st) : fstat (fd, &st))
@@ -257,20 +246,6 @@ fdutimens (int fd, char const *file, struct timespec const timespec[2])
 # if HAVE_UTIMENSAT
       if (fd < 0)
         {
-#  if defined __APPLE__ && defined __MACH__
-          size_t len = strlen (file);
-          if (len > 0 && file[len - 1] == '/')
-            {
-              struct stat statbuf;
-              if (stat (file, &statbuf) < 0)
-                return -1;
-              if (!S_ISDIR (statbuf.st_mode))
-                {
-                  errno = ENOTDIR;
-                  return -1;
-                }
-            }
-#  endif
           result = utimensat (AT_FDCWD, file, ts, 0);
 #  ifdef __linux__
           /* Work around a kernel bug:
@@ -414,10 +389,10 @@ fdutimens (int fd, char const *file, struct timespec const timespec[2])
     struct timeval *t;
     if (ts)
       {
-        timeval[0] = (struct timeval) { .tv_sec  = ts[0].tv_sec,
-                                        .tv_usec = ts[0].tv_nsec / 1000 };
-        timeval[1] = (struct timeval) { .tv_sec  = ts[1].tv_sec,
-                                        .tv_usec = ts[1].tv_nsec / 1000 };
+        timeval[0].tv_sec = ts[0].tv_sec;
+        timeval[0].tv_usec = ts[0].tv_nsec / 1000;
+        timeval[1].tv_sec = ts[1].tv_sec;
+        timeval[1].tv_usec = ts[1].tv_nsec / 1000;
         t = timeval;
       }
     else
@@ -449,7 +424,7 @@ fdutimens (int fd, char const *file, struct timespec const timespec[2])
 #  endif
         if (futimes (fd, t) == 0)
           {
-#  if defined __linux__ && defined __GLIBC__
+#  if __linux__ && __GLIBC__
             /* Work around a longstanding glibc bug, still present as
                of 2010-12-27.  On older Linux kernels that lack both
                utimensat and utimes, glibc's futimes rounds instead of
@@ -511,8 +486,8 @@ fdutimens (int fd, char const *file, struct timespec const timespec[2])
       struct utimbuf *ut;
       if (ts)
         {
-          utimbuf = (struct utimbuf) { .actime  = ts[0].tv_sec,
-                                       .modtime = ts[1].tv_sec };
+          utimbuf.actime = ts[0].tv_sec;
+          utimbuf.modtime = ts[1].tv_sec;
           ut = &utimbuf;
         }
       else
@@ -528,20 +503,8 @@ fdutimens (int fd, char const *file, struct timespec const timespec[2])
    TIMESPEC[0] and TIMESPEC[1], respectively.  */
 int
 utimens (char const *file, struct timespec const timespec[2])
-#undef utimens
 {
-#if HAVE_UTIMENS
-  /* NetBSD's native utimens() does not fulfil the Gnulib expectations:
-     At least in NetBSD 10.0, it does not validate the timespec argument.  */
-  if (timespec != NULL && !is_valid_timespecs (timespec))
-    {
-      errno = EINVAL;
-      return -1;
-    }
-  return utimens (file, timespec);
-#else
   return fdutimens (-1, file, timespec);
-#endif
 }
 
 /* Set the access and modification timestamps of FILE to be
@@ -550,18 +513,7 @@ utimens (char const *file, struct timespec const timespec[2])
    changing symlink timestamps, but FILE was a symlink.  */
 int
 lutimens (char const *file, struct timespec const timespec[2])
-#undef lutimens
 {
-#if HAVE_LUTIMENS
-  /* NetBSD's native lutimens() does not fulfil the Gnulib expectations:
-     At least in NetBSD 10.0, it does not validate the timespec argument.  */
-  if (timespec != NULL && !is_valid_timespecs (timespec))
-    {
-      errno = EINVAL;
-      return -1;
-    }
-  return lutimens (file, timespec);
-#else
   struct timespec adjusted_timespec[2];
   struct timespec *ts = timespec ? adjusted_timespec : NULL;
   int adjustment_needed = 0;
@@ -581,11 +533,11 @@ lutimens (char const *file, struct timespec const timespec[2])
      fdutimens' worry about buggy NFS clients.  But we do have to
      worry about bogus return values.  */
 
-# if HAVE_UTIMENSAT
+#if HAVE_UTIMENSAT
   if (0 <= lutimensat_works_really)
     {
       int result;
-#  if defined __linux__ || defined __sun || defined __NetBSD__
+# if __linux__ || __sun
       /* As recently as Linux kernel 2.6.32 (Dec 2009), several file
          systems (xfs, ntfs-3g) have bugs with a single UTIME_OMIT,
          but work if both times are either explicitly specified or
@@ -595,7 +547,6 @@ lutimens (char const *file, struct timespec const timespec[2])
          UTIME_OMIT would have worked.
 
          The same bug occurs in Solaris 11.1 (Apr 2013).
-         The same bug occurs in NetBSD 10.0 (May 2024).
 
          FIXME: Simplify this for Linux in 2016 and for Solaris in
          2024, when file system bugs are no longer common.  */
@@ -610,9 +561,9 @@ lutimens (char const *file, struct timespec const timespec[2])
           /* Note that st is good, in case utimensat gives ENOSYS.  */
           adjustment_needed++;
         }
-#  endif
+# endif
       result = utimensat (AT_FDCWD, file, ts, AT_SYMLINK_NOFOLLOW);
-#  ifdef __linux__
+# ifdef __linux__
       /* Work around a kernel bug:
          https://bugzilla.redhat.com/show_bug.cgi?id=442352
          https://bugzilla.redhat.com/show_bug.cgi?id=449910
@@ -622,7 +573,7 @@ lutimens (char const *file, struct timespec const timespec[2])
          are no longer in common use.  */
       if (0 < result)
         errno = ENOSYS;
-#  endif
+# endif
       if (result == 0 || errno != ENOSYS)
         {
           utimensat_works_really = 1;
@@ -631,7 +582,7 @@ lutimens (char const *file, struct timespec const timespec[2])
         }
     }
   lutimensat_works_really = -1;
-# endif /* HAVE_UTIMENSAT */
+#endif /* HAVE_UTIMENSAT */
 
   /* The platform lacks an interface to set file timestamps with
      nanosecond resolution, so do the best we can, discarding any
@@ -647,17 +598,17 @@ lutimens (char const *file, struct timespec const timespec[2])
 
   /* On Linux, lutimes is a thin wrapper around utimensat, so there is
      no point trying lutimes if utimensat failed with ENOSYS.  */
-# if HAVE_LUTIMES && !HAVE_UTIMENSAT
+#if HAVE_LUTIMES && !HAVE_UTIMENSAT
   {
     struct timeval timeval[2];
     struct timeval *t;
     int result;
     if (ts)
       {
-        timeval[0] = (struct timeval) { .tv_sec = ts[0].tv_sec,
-                                        .tv_usec = ts[0].tv_nsec / 1000 };
-        timeval[1] = (struct timeval) { .tv_sec = ts[1].tv_sec,
-                                        .tv_usec = ts[1].tv_nsec / 1000 };
+        timeval[0].tv_sec = ts[0].tv_sec;
+        timeval[0].tv_usec = ts[0].tv_nsec / 1000;
+        timeval[1].tv_sec = ts[1].tv_sec;
+        timeval[1].tv_usec = ts[1].tv_nsec / 1000;
         t = timeval;
       }
     else
@@ -667,7 +618,7 @@ lutimens (char const *file, struct timespec const timespec[2])
     if (result == 0 || errno != ENOSYS)
       return result;
   }
-# endif /* HAVE_LUTIMES && !HAVE_UTIMENSAT */
+#endif /* HAVE_LUTIMES && !HAVE_UTIMENSAT */
 
   /* Out of luck for symlinks, but we still handle regular files.  */
   if (!(adjustment_needed || REPLACE_FUNC_STAT_FILE) && lstat (file, &st))
@@ -676,5 +627,4 @@ lutimens (char const *file, struct timespec const timespec[2])
     return fdutimens (-1, file, ts);
   errno = ENOSYS;
   return -1;
-#endif
 }
